@@ -48,8 +48,18 @@ def on_execution_path(m):
     case of WaitUntil, PauseFor and When"""
     @functools.wraps(m)
     def wrapper(self, task, *args, **kwargs):
-        return None if task is None else m(self, task, *args, **kwargs)
+        try:
+            return None if task is None else m(self, task, *args, **kwargs)
+        except Exception as err:
+            if isinstance(err, ImproperlyConfigured):
+                raise err
+            if task is not None:
+                self.store_result(task, repr(err), dict(error=traceback.format_exc()))
+                self.release_lock(task)
+                self._automation._db.finished = True
+                self._automation._db.save()
 
+            return None
     return wrapper
 
 
@@ -114,6 +124,16 @@ class Node:
         task.save()
         return None
 
+    @staticmethod
+    def store_result(task: models.AutomationTaskModel, message, result):
+        task.message = message[0:settings.MAX_FIELD_LENGTH]
+        try:  # Check if result is json serializable
+            json.dumps(result)  # Raise error if not json-serializable
+            task.result = result  # if it is, store it
+        except TypeError:
+            task.result = None
+        task.save()
+
     def leave(self, task: models.AutomationTaskModel):
         if task is not None:
             task.finished = now()
@@ -123,7 +143,7 @@ class Node:
             else:
                 next_node = self._next
             if next_node is None:
-                raise ImproperlyConfigured("No End statement")
+                raise ImproperlyConfigured("No End() node after %s" % self._name)
             return next_node
 
     @on_execution_path
@@ -337,23 +357,18 @@ class Execute(Node):
             self._err = None
             try:
                 result = self.method(task, *args, **kwargs)
-                task.message = "OK"
-                try:  # Check if result is json serializable
-                    json.dumps(result)  # Raise error if not json-serializable
-                    task.result = result  # if it is, store it
-                except TypeError:
-                    task.result = None
+                self.store_result(task, "OK", result)
             except Exception as err:
                 if isinstance(err, ImproperlyConfigured):
                     raise err
                 self._err = err
-                task.message = repr(err)[:settings.MAX_FIELD_LENGTH]
-                task.result = dict(error=traceback.format_exc())
+                self.store_result(task, repr(err), dict(error=traceback.format_exc()))
+
 
         if self.args is not None and len(self.args) > 0:  # Empty arguments: No-op
             args = (self.resolve(value) for value in self.args)
             kwargs = {key: self.resolve(value) for key, value in self.kwargs.items()}
-            if kwargs.pop("threaded", False):
+            if kwargs.get("threaded", False):
                 assert self._on_error is None, "No .OnError statement on threaded executions"
                 threading.Thread(target=func, args=[task] + list(args), kwargs=kwargs).start()
             else:
@@ -362,7 +377,7 @@ class Execute(Node):
                     if self._on_error:
                         self._next = self._on_error
                     else:
-                        self.leave(task)
+                        self.release_lock(task)
                         self._automation._db.finished = True
                         self._automation._db.save()
                         return None
@@ -445,6 +460,7 @@ class Form(Node):
             task.interaction_user = self.get_user()
             task.interaction_group = self.get_group()
             if task.data.get(f"_{self._name}_validated", None) is None:
+                task.requires_interaction = True
                 self.release_lock(task)  # Release lock and stop automation until form is validated
                 return None
         return task  # Continue with validated form
@@ -504,6 +520,39 @@ class ModelForm(Form):
             return dict()
 
 
+class SendMessage(Node):
+    def __init__(self, target, message, token=None, allow_multiple_receivers=False, **kwargs):
+        self._target = target
+        self._message = message
+        self._token = token
+        self._allow_multiple_receivers = allow_multiple_receivers
+        self.kwargs = kwargs
+        super().__init__()
+
+    @on_execution_path
+    def send_handler(self, task):
+        cls = self._target
+        if isinstance(cls, str):
+            cls = models.get_automation_class(cls)
+        if issubclass(cls, Automation):
+            results = cls.broadcast_message(self._message, self._token, data=self.kwargs)
+        elif isinstance(cls, Automation) or isinstance(cls, int):
+            results = [cls.dispatch_message(self._message, self._token, data=self.kwargs)]
+        else:
+            raise ImproperlyConfigured("")
+        self.store_result(task, "OK", dict(results=results))
+        return task
+
+    def execute(self, task: models.AutomationTaskModel):
+        task = super().execute(task)
+        return self.send_handler(task)
+
+#
+# Automation class
+#
+#
+
+
 def on_signal(signal, start=None, **kwargs):
     """decorator for automations to connect to Django signals"""
     def decorator(cls):
@@ -514,9 +563,7 @@ def on_signal(signal, start=None, **kwargs):
 
 class Automation:
     model_class = models.AutomationModel
-    singleton = False
-
-    end = End()  # Endpoint of automation
+    unique = False
 
     def __init__(self, **kwargs):
         super().__init__()
@@ -538,12 +585,16 @@ class Automation:
                 elif not isinstance(kwargs[name], int):  # Convert instance to id
                     kwargs[name] = kwargs[name].id
         self._iter[prev] = None  # Last item
-
+        autorun = kwargs.pop("autorun", True)
         if 'automation' in kwargs:
             self._db = kwargs.pop('automation')
+            autorun = False
             assert isinstance(self._db, models.AutomationModel), \
                 "automation= parameter needs to be AutomationModel instance"
+            assert not kwargs, "Too many arguments for automation %s. " \
+                               "If 'automation' is given, no parameters allowed" % self.__clas__.__name__
         elif 'automation_id' in kwargs:  # Attach to automation in DB
+            self._create_model_properties(kwargs)
             self._db, _ = self.model_class.objects.get_or_create(
                 id=kwargs.pop('automation_id'),
                 defaults=dict(
@@ -551,19 +602,59 @@ class Automation:
                     finished=False,
                     data=kwargs,
                 ))
-        elif self.singleton:  # Create or get singleton in DB
-            self._db, _ = self.model_class.objects.create_or_get(
+            autorun = False
+            assert not kwargs, "Too many arguments for automation %s. " \
+                               "If 'automation' is given, no parameters allowed" % self.__clas__.__name__
+        elif self.unique is True:  # Create or get singleton in DB
+            self._db, created = self.model_class.objects.get_or_create(
                 automation_class=self.get_automation_class_name(),
             )
-            self._db.data = kwargs
-            self._db.finished = False
-            self._db.save()
-        else:  # Create new automation in DB
+            if created:
+                self._db.data = kwargs
+                self._db.finished = False
+                self._db.save()
+            else:
+                assert not kwargs, "Too many arguments for automation %s. " \
+                                   "If 'automation' is given, no parameters allowed" % self.__clas__.__name__
+        elif self.unique:
+            assert isinstance(self.unique, (list, tuple)), ".singleton can be bool, list, tuple or None"
+            for key in self.unique:
+                assert key not in ("automation", "automation_id", "autorun"), \
+                    f"'{key}' cannot be parameter to distinguish unique automations. Chose a different name."
+                assert key in kwargs, "to ensure unqiue property, " \
+                                      "create automation with '%s=...' parameter" % key
+            qs = self.model_class.objects.filter(finished=False)
+            for instance in qs:
+                identical = sum((0 if key not in instance.data or instance.data[key] != kwargs[key] else 1
+                                 for key in self.unique))
+                if identical == len(self.unique):
+                    self._db = instance
+                    break
+            else:
+                self._create_model_properties(kwargs)
+                self._db = self.model_class.objects.create(
+                    automation_class=self.get_automation_class_name(),
+                    finished=False,
+                    data=kwargs,
+                )
+        else:
+            self._create_model_properties(kwargs)
             self._db = self.model_class.objects.create(
                 automation_class=self.get_automation_class_name(),
                 finished=False,
                 data=kwargs,
             )
+        assert self._db is not None, "Internal error"
+        if autorun:
+            self.run()
+
+    def _create_model_properties(self, kwargs):
+        for name, value in kwargs.items():
+            if isinstance(value, ModelBase):
+                model_class = value.__class__
+                setattr(self, name,  # Replace property by get_model_instance
+                        property(lambda slf: slf.get_model_instance(model_class, name), self))
+                kwargs[name] = kwargs[name].id
 
     def get_model_instance(self, model, name):
         if not hasattr(self, '_' + name):
@@ -629,29 +720,77 @@ class Automation:
     def finished(self):
         return self._db.finished
 
-    def start_from_request(self, request):              # pragma: no cover
-        pass
-
-    def send_message(self, message, token, request): # pragma: no cover
+    def send_message(self, message, token, data=None):
         """RECEIVES message and dispatches it within the class
         Called send_message so that sending a message to an automation
         is `automation.send_message(...)"""
-        if hasattr(self, "receive_"+message):
+        if self.__class__.satisfies_data_requirements(message, data):
             method = getattr(self, "receive_"+message)
-            return method(token, request)
+            return method(token, data)
         return None
 
-    def kill_automation(self):
+    @classmethod
+    def satisfies_data_requirements(cls, message, get):
+        if hasattr(cls, "receive_" + message):
+            method = getattr(cls, "receive_" + message)
+            if not hasattr(method, "data_requirements"):
+                return True
+            accessor = get.GET if hasattr(get, 'GET') else get
+            for param, type_class in method.data_requirements.items():
+                if param not in accessor:
+                    return False
+                if not isinstance(accessor[param], type_class):  # Try simple conversion
+                    try:
+                        type_class(accessor[param])
+                    except (ValueError, TypeError):
+                        return False
+            return True
+        return False
+
+    def kill(self):
+        """Deletes the automation instance in models.AutomationModel"""
         self._db.delete()
+        self._db = None
+
 
     @classmethod
-    def dispatch_message(cls, automation, message, token, request):
-        if isinstance(automation, int):
-            automation = cls(automation_id=automation)
-        assert isinstance(automation, cls), f"Wrong class to dispatch message: " \
-                                            f"{automation.__class__.__name__} found, " \
-                                            f"{cls.__name__} expected"
-        return automation.send_message(message, token, request)
+    def dispatch_message(cls, automation, message, token, data):
+        if cls.satisfies_data_requirements(message, data):
+            if isinstance(automation, int):
+                automation = cls(automation_id=automation)
+            assert isinstance(automation, cls), f"Wrong class to dispatch message: " \
+                                                f"{automation.__class__.__name__} found, " \
+                                                f"{cls.__name__} expected"
+            return automation.send_message(message, token, data)
+
+    @classmethod
+    def broadcast_message(cls, message, token, data):
+        results = []
+        if cls.satisfies_data_requirements(message, data):
+            for automation in models.AutomationModel.objects.filter(
+                    finished=False,
+                    automation_class=cls.__module__+"."+cls.__name__,
+            ):
+                automation = cls(automation=automation)
+                result = automation.send_message(message, token, data)
+                results.append(result)
+                if isinstance(result, str) and result == "received":
+                    break
+        return results
+
+    @classmethod
+    def create_on_message(cls, message, token, data):
+        if cls.satisfies_data_requirements(message, data):
+            kwargs = dict()
+            accessor = data.GET if hasattr(data, 'GET') else data
+            if isinstance(cls.unique, (list, tuple)):
+                for param in cls.unique:
+                    if param in accessor:
+                        kwargs[param] = accessor.get(param)
+            instance = cls(autorun=False, **kwargs)
+            instance.send_message(message, token, data)
+            return instance
+        return None
 
     @classmethod
     def on(cls, signal, start=None, **kwargs):
@@ -666,7 +805,6 @@ class Automation:
             instance.started_by_signal(sender, kwargs)  # initialize based on sender data
         instance.run(None, None if start is None else getattr(instance, start))  # run
 
-
     def __str__(self):
         return self.__class__.__name__
 
@@ -697,3 +835,11 @@ def get_automations(app=None):
 
     return automation_list
 
+
+def require_data_parameters(**kwargs):
+    """decorates Automation class receiver methods to set the data_requirement attribute
+    It is checked by cls.satisfies_data_requirements"""
+    def decorator(method):
+        method.data_requirements = kwargs
+        return method
+    return decorator
