@@ -3,9 +3,8 @@
 import sys
 from logging import getLogger
 
-from django import forms
-from django.contrib.auth.models import Permission, User
-from django.db.models import Q, QuerySet
+from django.contrib.auth.models import User
+from django.db.models import Q
 from django.utils.module_loading import import_string
 from django.utils.timezone import now
 from django.utils.translation import gettext as _
@@ -77,6 +76,9 @@ class AutomationModel(models.Model):
                 automation.save()
                 logger.error(f'Error: {repr(e)}', exc_info=sys.exc_info())
 
+    def __str__(self):
+        return f"<AutomationModel for {self.automation_class}>"
+
 
 class AutomationTaskModel(models.Model):
     automation = models.ForeignKey(
@@ -98,6 +100,10 @@ class AutomationTaskModel(models.Model):
             default=0,
             verbose_name=_("Locked"),
     )
+    requires_interaction = models.BooleanField(
+        default=False,
+        verbose_name=_("Requires interaction")
+    )
     interaction_user = models.ForeignKey(
             'auth.User',
             null=True,
@@ -159,5 +165,5 @@ class AutomationTaskModel(models.Model):
     @classmethod
     def get_open_tasks(cls, user):
         candidates = cls.objects.filter(finished=None)
-        return tuple(task for task in candidates if user in task.get_users_with_permission())
-
+        return tuple(task for task in candidates
+                     if task.requires_interaction and user in task.get_users_with_permission())
