@@ -20,8 +20,6 @@ from django.test import RequestFactory
 
 # Create your tests here.
 
-test_signal = django.dispatch.Signal()
-
 
 class Print(flow.Execute):
     @staticmethod
@@ -30,7 +28,7 @@ class Print(flow.Execute):
 
 
 class TestAutomation(flow.Automation):
-    start =             flow.Execute(this.init, threaded=True).AsSoonAs(lambda x:True).AsSoonAs(this.cont)
+    start =             flow.Execute(this.init).AsSoonAs(lambda x: True).AsSoonAs(this.cont)
     intermediate =      flow.Execute("self.init2")
     func_if =           flow.If(lambda x: x.data['more_participants'] == "test").Then().Else(this.print)
     if_clause =         flow.If(lambda x: x.data['more_participants'] == "test").Then("self.conditional")
@@ -41,7 +39,7 @@ class TestAutomation(flow.Automation):
 
     conditional =       flow.Execute(this.init2).Next("self.if2")
 
-    def init(self, task_instance, *args, **kwargs):
+    def init(self, task, *args, **kwargs):
         if 'participants' not in self.data:
             self.data['participants'] = []
             self.save()
@@ -52,13 +50,12 @@ class TestAutomation(flow.Automation):
         return task  # Illegal since not json serializable
 
     def cont(self, task):
-        return bool(self) and (task)  # True
+        return bool(self) and bool(task)  # True
 
     def print(self, task):
         print("Hello", task.data)
 
 
-
 class TestForm(forms.Form):
     success_url = "https://www.google.de/"
     first_name = forms.CharField(
@@ -91,14 +88,6 @@ class TestSplitJoin(flow.Automation):
     t30 = Print("Thread 30").Next(this.join)
 
 
-@flow.on_signal(test_signal)
-class SignalAutomation(flow.Automation):
-    def started_by_signal(self, *args, **kwargs):
-        self.data['started'] = 'yeah!'
-        self.save()
-    start = flow.End()
-
-
 class AtmTaskForm(forms.ModelForm):
     class Meta:
         model = models.AutomationTaskModel
@@ -114,7 +103,7 @@ class FormTest(flow.Automation):
     form = (flow.Form(TestForm, context=dict(claim="Save"))
             .User(id=0))
     form2 = (flow.Form(TestForm, context=dict(claim="Save")).User(id=0)
-            .Permission("automations.create_automationmodel"))
+             .Permission("automations.create_automationmodel"))
     end = flow.End()
 
     def init_with_item(self, task_instance):
@@ -142,10 +131,25 @@ class BoundToFail(flow.Automation):
     error_node = Print("Oh dear").Next(this.not_caught)
 
 
+class SingletonAutomation(flow.Automation):
+    unique = True
+
+    end = flow.End()
+
+
+class ByEmailSingletonAutomation(flow.Automation):
+    unique = ("email",)
+
+    end = flow.End()
+
+    @flow.require_data_parameters(email=str, mails=int)
+    def receive_test(self, token, data):
+        pass
+
 class ModelTestCase(TestCase):
 
     def test_modelsetup(self):
-        x = TestAutomation()
+        x = TestAutomation(autorun=False)
         qs = models.AutomationModel.objects.all()
         self.assertEqual(len(qs), 1)
         self.assertEqual(qs[0].automation_class, 'automations.tests.TestAutomation')
@@ -167,7 +171,6 @@ class AutomationTestCase(TestCase):
     def test_split_join(self):
         with patch('sys.stdout', new=StringIO()) as fake_out:
             atm = TestSplitJoin()
-            atm.run()
         output = fake_out.getvalue().splitlines()
         self.assertEqual(output[0], "start Hello, this is the single thread")
         self.assertEqual(output[1], "l10 Line 10")
@@ -188,7 +191,7 @@ class FormTestCase(TestCase):
             username='jacob', email='jacob@…', password='top_secret')
 
     def test_form(self):
-        atm = FormTest()
+        atm = FormTest(autorun=False)
         atm.form._user=dict(id=self.user.id)  # Fake User
         atm.form2._user=dict(id=self.user.id)  # Fake User
         atm.run()
@@ -229,6 +232,30 @@ class FormTestCase(TestCase):
         self.assertEqual(len(atm.form2.get_users_with_permission()), 0)
 
 
+
+test_signal = django.dispatch.Signal()
+
+
+@flow.on_signal(test_signal)
+class SignalAutomation(flow.Automation):
+    def started_by_signal(self, *args, **kwargs):
+        self.data['started'] = 'yeah!'
+        self.save()
+    start = flow.Execute().AfterPausingFor(datetime.timedelta(days=1))
+    end = flow.End()
+
+    def receive_new_user(self, token, data=None):
+        self.data['token'] = token
+        self.save()
+        return "received"
+
+
+class SendMessageAutomation(flow.Automation):
+    start = flow.SendMessage(SignalAutomation, "new_user", "12345678")
+    to_nowhere = flow.SendMessage("automations.tests.FormTest", "this_receiver_does_not_exist")
+    end = flow.End()
+
+
 class SignalTestCase(TestCase):
     def test_signal(self):
         self.assertEqual(0, len(models.AutomationModel.objects.filter(
@@ -242,6 +269,12 @@ class SignalTestCase(TestCase):
         )
         self.assertEqual(1, len(inst))
         self.assertEqual(inst[0].data.get("started", ""), "yeah!")
+        SendMessageAutomation()
+        inst = models.AutomationModel.objects.filter(
+            automation_class='automations.tests.SignalAutomation',
+        )
+        self.assertEqual(1, len(inst))
+        self.assertEqual(inst[0].data.get("token", ""), "12345678")
 
         self.assertGreater(len(inst[0].automationtaskmodel_set.all()), 0)
 
@@ -249,15 +282,12 @@ class SignalTestCase(TestCase):
 class RepeatTest(TestCase):
     def test_repeat(self):
         atm = Looping()
-        atm.run()
         tasks = atm._db.automationtaskmodel_set.filter(finished=None)
         self.assertEqual(len(tasks), 3)
         atm.run()
         tasks = atm._db.automationtaskmodel_set.filter(finished=None)
         self.assertEqual(len(tasks), 3)
 
-
-
     def test_get_automations(self):
         self.assertEqual(len(flow.get_automations()), 0)
         self.assertEqual(len(flow.get_automations("automations.flow")), 1)
@@ -282,7 +312,7 @@ class ManagementCommandTest(TestCase):
         self.assertGreater(len(models.AutomationTaskModel.objects.filter(
             automation_id=db_id
         )), 0)
-        atm.kill_automation()
+        atm.kill()
         self.assertEqual(len(models.AutomationTaskModel.objects.filter(
             automation_id=db_id
         )), 0)
@@ -292,9 +322,99 @@ class ExecutionErrorTest(TestCase):
     def test_managment_command(self):
         with patch('sys.stdout', new=StringIO()) as fake_out:
             atm = BoundToFail()
-            atm.run()
         output = fake_out.getvalue().splitlines()
         self.assertEqual(output[0], "start Will divide by zero.")
         self.assertEqual(output[-1], "error_node Oh dear")
         self.assertNotIn("never This should NOT be printed", output)
         self.assertNotIn("never_reached Will not reach this", output)
+
+
+class SingletonTest(TestCase):
+    def test_singleton(self):
+        inst1 = SingletonAutomation(autorun=False)
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class="automations.tests.SingletonAutomation"
+        )), 1)
+        inst2 = SingletonAutomation(autorun=False)
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class="automations.tests.SingletonAutomation"
+        )), 1)
+        self.assertEqual(inst1._db, inst2._db)
+        self.assertNotEqual(inst1, inst2)
+
+    def test_rel_singleton(self):
+        inst1 = ByEmailSingletonAutomation(email="none@nowhere.com", autorun=False)
+        atm_name = inst1.get_automation_class_name()
+        self.assertEqual(atm_name.rsplit(".", 1)[-1], inst1.end.get_automation_name())
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class=atm_name
+        )), 1)
+        inst2 = ByEmailSingletonAutomation(email="nowhere@none.com", autorun=False)
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class=atm_name
+        )), 2)
+        self.assertNotEqual(inst1._db, inst2._db)
+        inst3 = ByEmailSingletonAutomation(email="nowhere@none.com", autorun=False)
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class=atm_name
+        )), 2)
+        self.assertEqual(inst2._db, inst3._db)
+
+        self.assertTrue(ByEmailSingletonAutomation.satisfies_data_requirements("test", dict(email="test", mails="2")))
+        self.assertTrue(ByEmailSingletonAutomation.satisfies_data_requirements("test", dict(email="test", mails=2)))
+        self.assertFalse(ByEmailSingletonAutomation.satisfies_data_requirements("test", dict(email="test", mails="t2")))
+        self.assertFalse(ByEmailSingletonAutomation.satisfies_data_requirements("test", dict(mails="t2")))
+        self.assertFalse(ByEmailSingletonAutomation.satisfies_data_requirements("nonexistent", dict(mails="t2")))
+
+        ByEmailSingletonAutomation.create_on_message("test", None, dict(email="new", mails=2))
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class="automations.tests.ByEmailSingletonAutomation"
+        )), 3)
+
+        ByEmailSingletonAutomation.create_on_message("test", None, dict(email="also_new"))
+        self.assertEqual(len(models.AutomationModel.objects.filter(
+            automation_class="automations.tests.ByEmailSingletonAutomation"
+        )), 3)
+
+        models.AutomationModel.run()
+
+
+class BogusAutomation1(flow.Automation):
+    start = flow.Execute(this.test).OnError(this.error)
+    end = flow.End()
+    error = flow.Execute(this.test)
+
+    def test(self, task):
+        raise SyntaxError("Darn, this is not good")
+
+    def time(self, task):
+        return True  # not a datetime
+
+
+class BogusAutomation2(flow.Automation):
+    start = flow.Execute(this.test)
+    mid = flow.Execute(this.test).AfterWaitingUntil(this.time)
+    end = flow.End().AfterWaitingUntil(this.time)
+
+    def test(self, task):
+        return "Truth"
+
+    def time(self, task):
+        return True  # not a datetime
+
+
+class ErrorTest(TestCase):
+    def test_errors(self):
+        atm = BogusAutomation1()
+        self.assertTrue(atm.finished())
+        self.assertEqual(len(atm._db.automationtaskmodel_set.all()), 2)
+        self.assertEqual(atm._db.automationtaskmodel_set.all()[1].message,
+                         "SyntaxError('Darn, this is not good')")
+
+        atm = BogusAutomation2()
+        self.assertTrue(atm.finished())
+        self.assertEqual(len(atm._db.automationtaskmodel_set.all()), 2)
+        self.assertEqual(atm._db.automationtaskmodel_set.all()[0].result,
+                         'Truth')
+        self.assertEqual(atm._db.automationtaskmodel_set.all()[1].message,
+                         "TypeError(\"'<' not supported between instances of 'bool' and 'datetime.datetime'\")")
