@@ -1,11 +1,14 @@
 # coding=utf-8
 
 # Create your views here.
-from django.contrib.auth.mixins import LoginRequiredMixin
+import datetime
+
+from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin
 from django.core.exceptions import PermissionDenied
 from django.http import Http404
 from django.shortcuts import get_object_or_404, redirect
 from django.urls import reverse
+from django.utils.timezone import now
 from django.views.generic import TemplateView, FormView
 from django.forms import BaseForm
 
@@ -82,9 +85,27 @@ class TaskListView(LoginRequiredMixin, TemplateView):
         return dict(error="", tasks=qs, count=len(qs))
 
 
-class TaskDashboardView(TemplateView):
+class UserIsStaff(UserPassesTestMixin):
+    def test_func(self):
+        return self.request.user.is_staff
+
+
+class TaskDashboardView(UserIsStaff, TemplateView):
     template_name = 'automations/dashboard.html'
 
     def get_context_data(self, **kwargs):
-        return dict(automations=models.AutomationModel.objects.all(),
-                    tasks=models.AutomationTaskModel.objects.all())
+        qs = models.AutomationModel.objects.filter(created__gt=now()-datetime.timedelta(days=30))
+        automations = []
+        for item in qs.order_by("automation_class").values("automation_class").distinct():
+            automation = models.get_automation_class(item['automation_class'])
+            qs_filtered = qs.filter(**item)
+            dashboard = (automation.get_dashboard_context(qs_filtered)
+                         if hasattr(automation, "get_dashboard_context") else dict())
+            automations.append(dict(cls=item['automation_class'],
+                                    verbose_name=automation.get_verbose_name(),
+                                    verbose_name_plural=automation.get_verbose_name_plural(),
+                                    running=qs_filtered.filter(finished=False),
+                                    finished=qs_filtered.filter(finished=True),
+                                    dashboard_template= getattr(automations, "dashboard_template", ""),
+                                    dashboard=dashboard))
+        return dict(automations=automations)
